; cmucs.ml
;
; [SJF] This is really not very useful unless you are willing to
; handle ^Z as a character prefix.  Some of the commands internally
; are quite handy, though.  This might be best for somebody who
; doesn't know Emacs yet.
;
; rearrange the character set.
;
; mostly compatible with <agin.emacs>cmucs at CMU-20C
;
; There are five flavors of commands:  ^x is the usual, esc-x is the super,
;    and ^Z ^x is a super-duper.  ^X ^x commands usually deal with files and
;    windows.  ^X x commands are rare.  There are no useful commands that use
;    capital letters or esc-control.
;
; Modifications to functions have been made in the following areas:  (See
;	[r]/usr/agin/emacs/cmucs.txt for a more complete description)
; Killing:  Uses killring.ml
; Searching:  character searches and zap to character
; Case shifts, transpose word and char behave differently at end of line
;   than at beginning.
; Word moves and end-of-word moves are independent of direction of motion.
;
; functions here (documented by S. Friedl @ Kent State)
;
; select-buffer			If this function has an argument,
;				if switches to the prompted-for buffer.
;				Otherwise it pops to it.
;
; character-search		Like ^S but it only goes after a single
;				character.  This character is gotten
;				directly from the tty rather than in the
;				minibuffer.
;
; reverse-character-search	Reverse version of above.
;
; zap-to-character		sets a mark, prompts for a character,
;				searches forward for it, and kills all
;				text in the region, not including the
;				searched character.
;
; zap-thru-character		same as above, but it also kills the
;				searched character.
;
; set-fixed-mark		set a mark that isn't part of the killring
;				system.  Only works when given a prefix.
;
; goto-fixed-mark		goes to the mark set by the set-fixed-mark
;				command.  Must have a prefix argument.
;
; forward-word-beginning	just like forward-word, but the cursor
;				is placed at the beginning of the word
;				instead of at the end.
;
; swap-word-boundaries		"enables" the forward-word-beginning and
;				kill-to-work-beginning functions to the
;				keys ESC-F and ESC-D
;
; unswap-word-boundaries	"disables" the above functions.  It
;				binds the normal forward-word and
;				kill-to-word-end functions to ESC-F
;				and ESC-D, respectively.
;
; scroll-smart			If there is a prefix-argument, it scrolls
;				that amount.  Otherwise it moves to the
;				next page.
;
; unscroll-smart		Reverse version of the above.
;
; cmucs-rebind			Does all the internal rebinding
;				needed to set up the CMU package.
;
; ***
; key bindings:
;
; ^Z(		start-remembering
; ^Z)		stop-remembering
; ^X<		shrink-window
; ESC-=		display-line-number
; ^Z=		srccom
; ^X>		enlarge-window
; ^Z]		match-begin
; ESC-_		backward-column
; ESC-~		not-modified-buffer
; ESC-a		backward-sentence
; ^Z^A		append-next-kill
; ^Z^B		backward-word-end
; ^X^B		select-buffer
; ESC-c		centre-line
; ^C		control-c
; ^X^C		control-c
; ^Z^C		control-c
; ^Xd		insert-date
; ^X^D		delete-window
; ^Z^E		execute-keyboard-macro
; ESC-g		goto-fixed-mark
; ^Xg		goto-tag
; ^Z^H		set-bounds
; ^Z^I		indent-region
; ESC-j		justify-paragraph
; ^Z^K		delete-region-to-buffer
; ^X^K		delete-buffer
; ^Z^L		goto-line
; ^X^L		list-buffers
; ESC-l		lower-smart
; ESC-^M	beginning-of-next-line
; ESC-m		set-fixed-mark
; ^X^O		next-window
; ^Z^P		prepend-region-to-buffer
; ^Z^Q		re-query-replace-string
; ^X^Q		set-read-only
; ^R		reverse-character-search
; ESC-r		reverse-incremental-search
; ^Z^R		re-reverse-search
; ^S		character-search
; ESC-s		incremental-search
; ^Z^S		re-search-forward
; ^T		transpose-smart
; ESC-t		transpose-word-smart
; ^Z^T		transpose-line
; ESC-u		capitalize-smart
; ^V		scroll-smart
; ESC-v		unscroll-smart
; ^Z^V		page-next-window
; ^Z^W		copy-region-to-buffer
; ^Z^Y		yank-buffer
; ESC-z		zap-to-character
; ^Z^Z		zap-thru-character
; ^X^Z		write-file-push
; ESC-^?	backward-kill-to-word-beginning
; ^Z-^?		backward-kill-to-word-end
; ESC-f		forward-word-beginning
; ^Z^F		forward-word
; ESC-d		kill-to-word-beginning
; ^Z^D		kill-to-word-end
;
(declare-global fixed-mark)
(declare-buffer-specific fixed-mark)
(declare-global csearch-default)

(defun
    (select-buffer
	(if prefix-argument-provided
	    (switch-to-buffer (arg 1 "switch-to-buffer: "))
	    (pop-to-buffer (arg 1 "pop-to-buffer: "))
	)
    )
    (character-search
	(search-forward (get-search-character)))
    (reverse-character-search
	(search-reverse (get-search-character)))
    (get-search-character
		newchar
	(setq newchar (get-tty-character))
	(if (& (!= newchar 18) (!= newchar 19))	; ^S or ^R repeat last search
	    (setq csearch-default (char-to-string newchar)))
	csearch-default)
    (zap-to-character
	(set-mark)
	(character-search)
	(backward-character)
	(kill-region))
    (zap-thru-character
	(set-mark)
	(character-search)
	(kill-region))
    (set-fixed-mark
	(if prefix-argument-provided
	    (setq fixed-mark (dot))
	    (set-mark)))
    (goto-fixed-mark
	(if prefix-argument-provided
	    (goto-character fixed-mark)
	    (exchange-dot-and-mark)))
    (forward-word-beginning
	(prefix-argument-loop
	    (while (& (! (eobp)) (looking-at "\\w"))
		   (forward-character))
	    (while (& (! (eobp)) (! (looking-at "\\w")))
		   (forward-character))
	)
    )
    (backward-word-end
	(backward-character)
	(prefix-argument-loop
	    (while (& (! (bobp)) (looking-at "\\w"))
		   (backward-character))
	    (while (& (! (bobp)) (! (looking-at "\\w")))
		   (backward-character))
	)
	(if (! (bobp)) (forward-character))
    )
    (kill-to-word-beginning
	(save-excursion
	    (set-mark)
	    (provide-prefix-argument prefix-argument (forward-word-beginning))
	    (kill-region)))
    (kill-to-word-end
	(save-excursion
	    (set-mark)
	    (provide-prefix-argument prefix-argument (forward-word))
	    (kill-region)))
    (backward-kill-to-word-beginning
	(save-excursion
	    (set-mark)
	    (provide-prefix-argument prefix-argument (backward-word))
	    (kill-region)))
    (backward-kill-to-word-end
	(save-excursion
	    (set-mark)
	    (provide-prefix-argument prefix-argument (backward-word-end))
	    (kill-region)))
    (swap-word-boundaries
	(bind-to-key "forward-word-beginning" "\ef")
	(bind-to-key "forward-word" "\^Z\^F")
	(bind-to-key "kill-to-word-beginning" "\ed")
	(bind-to-key "kill-to-word-end" "\^Z\^D"))
    (unswap-word-boundaries
	(bind-to-key "forward-word" "\ef")
	(bind-to-key "forward-word-beginning" "\^Z\^F")
	(bind-to-key "kill-to-word-end" "\ed")
	(bind-to-key "kill-to-word-beginning" "\^Z\^D"))
    (scroll-smart
	(if prefix-argument-provided
	    (provide-prefix-argument prefix-argument (scroll-one-line-up))
	    (next-page)))
    (unscroll-smart
	(if prefix-argument-provided
	    (provide-prefix-argument prefix-argument (scroll-one-line-down))
	    (previous-page)))
    )

(load		"killring.ml")
(setq-default &kill-lines-magic 1)

(setq-default &Default-Transpose-Magic 1)
(autoload "transpose-line"		"transp.ml")
(autoload "transpose-word"		"transp.ml")

(autoload "lower-smart"			"cmucs-smart.ml")
(autoload "upper-smart"			"cmucs-smart.ml")
(autoload "capitalize-smart"		"cmucs-smart.ml")
(autoload "transpose-smart"		"cmucs-smart.ml")
(autoload "transpose-word-smart"	"cmucs-smart.ml")

(autoload "backward-column"		"cmucs-misc.ml")
(autoload "display-line-number"		"cmucs-misc.ml")
(autoload "write-file-push"		"cmucs-misc.ml")
(autoload "write-file-pause"		"cmucs-misc.ml")
(autoload "dec-shell"			"cmucs-misc.ml")
(autoload "control-c"			"cmucs-misc.ml")
(autoload "beginning-of-next-line"	"cmucs-misc.ml")
(autoload "set-bounds"			"cmucs-misc.ml")
(autoload "insert-date"			"cmucs-misc.ml")
(autoload "match-begin"			"cmucs-misc.ml")

(autoload "set-read-only"		"cmucs-modes.ml")
(autoload "not-modified-buffer"		"cmucs-modes.ml")
(autoload "revert-file"			"cmucs-modes.ml")
(autoload "scribe-mode-hook"		"cmucs-modes.ml")

(autoload "centre-line"			"centre-line.ml")
(autoload "indent-region"		"ind-region.ml")
(autoload "srccom"			"srccom.ml")
(autoload "goto-tag"			"tags.ml")

(defun
    (bboard-hook
	(remove-local-binding "\^Z")
    )
)

(defun
    (escape-lowercase-equivalent
	(execute-binding
	    (concat "\e" (char-to-string (+ 32 (last-key-struck))))
	)
    )
    (control-X-lowercase-equivalent
	(execute-binding
	    (concat "\^X" (char-to-string (+ 32 (last-key-struck))))
	)
    )
)

(defun
    (cmucs-rebind
	(progn i
	    (setq i 'A')
	    (while (<= i 'Z')
		   (bind-to-key "escape-lowercase-equivalent"
		       (concat "\e" (char-to-string i)))
		   (bind-to-key "control-X-lowercase-equivalent"
		       (concat "\^X" (char-to-string i)))
		   (++ i)
;		   (setq i (+ 1 i))
	    )
	)
	(define-keymap "z-commands")
	(bind-to-key "z-commands" "\^Z")

	(if (error-occurred (start-remembering) (stop-remembering))
	    (progn
		  (bind-to-key "&start-remembering" "\^Z(")
		  (bind-to-key "&stop-remembering" "\^Z)")
	    )
	    (progn
		  (bind-to-key "start-remembering" "\^Z(")
		  (bind-to-key "stop-remembering" "\^Z)")
	    )
	)
	(bind-to-key "shrink-window"		"\^X<")
	(bind-to-key "display-line-number"	"\e=")
	(bind-to-key "srccom"			"\^Z=")
	(bind-to-key "enlarge-window"		"\^X>")
	(bind-to-key "match-begin"		"\^Z]")
	(bind-to-key "backward-column"		"\e_")
	(bind-to-key "not-modified-buffer"	"\e~")
	(bind-to-key "backward-sentence"	"\ea")
	(bind-to-key "append-next-kill"		"\^Z\^A")
	(bind-to-key "backward-word-end"	"\^Z\^B")
	(bind-to-key "select-buffer"		"\^X\^B")
	(bind-to-key "centre-line"		"\ec")
	(bind-to-key "control-c"		"\^C")
	(bind-to-key "control-c"		"\^X\^C")
	(bind-to-key "control-c"		"\^Z\^C")
	(bind-to-key "insert-date"		"\^Xd")
	(bind-to-key "delete-window"		"\^X\^D")
	(bind-to-key "execute-keyboard-macro"	"\^Z\^E")
	(bind-to-key "goto-fixed-mark"		"\eg")
	(bind-to-key "goto-tag"			"\^Xg")
	(bind-to-key "set-bounds"		"\^Z\^H")
	(bind-to-key "indent-region"		"\^Z\^I")
	(bind-to-key "justify-paragraph"	"\ej")
	(bind-to-key "delete-region-to-buffer"	"\^Z\^K")
	(bind-to-key "delete-buffer"		"\^X\^K")
	(bind-to-key "goto-line"		"\^Z\^L")
	(bind-to-key "list-buffers"		"\^X\^L")
	(bind-to-key "lower-smart"		"\el")
	(bind-to-key "beginning-of-next-line"	"\e\^M")
	(bind-to-key "set-fixed-mark"		"\em")
	(bind-to-key "next-window"		"\^X\^O")
	(bind-to-key "prepend-region-to-buffer"	"\^Z\^P")
	(bind-to-key "re-query-replace-string"	"\^Z\^Q")
	(bind-to-key "set-read-only"		"\^X\^Q")
	(bind-to-key "reverse-character-search"	"\^R")
	(bind-to-key "reverse-incremental-search" "\er")
	(bind-to-key "re-search-reverse"	"\^Z\^R")
	(bind-to-key "character-search"		"\^S")
	(bind-to-key "incremental-search"	"\es")
	(bind-to-key "re-search-forward"	"\^Z\^S")
 	(bind-to-key "transpose-smart"		"\^T")
	(bind-to-key "transpose-word-smart"	"\et")
	(bind-to-key "transpose-line"		"\^Z\^T")
	(bind-to-key "upper-smart"		"\eu")
	(bind-to-key "capitalize-smart"		"\^Z\^U")
	(bind-to-key "scroll-smart"		"\^V")
	(bind-to-key "unscroll-smart"		"\ev")
	(bind-to-key "page-next-window"		"\^Z\^V")
	(bind-to-key "copy-region-to-buffer"	"\^Z\^W")
	(bind-to-key "yank-buffer"		"\^Z\^Y")
	(bind-to-key "zap-to-character"		"\ez")
	(bind-to-key "zap-thru-character"	"\^Z\^Z")
	(bind-to-key "write-file-push"		"\^X\^Z")
	(bind-to-key "backward-kill-to-word-beginning" "\e\177")
	(bind-to-key "backward-kill-to-word-end" "\^Z\177")
	(swap-word-boundaries)
    )
)

(if (! (is-bound &cmucs-load-only))
    (cmucs-rebind)
)
