; dbman.ml	database maintenance functions
; 
; These functions are primitives that are used to manipulate databases
; using the ndbm package.  There are functions to generate a list of
; database keys in a buffer, and functions to perform a given action on
; the entries whose keys are in the list.  Hooks are provided for user
; code to control processing of the entries.  Defaults are assigned
; to the hooks here, but user code can define the hooks differently
; before calling the entry-processing functions.  The hooks can be
; restored to their default values by calling db-revert-hooks.
; 
(setq-default db-dir		(expand-file-name 
				  "/usr/lib/emacs/databases"))
(declare-global db-entry-buf)	; buffer for editing db entry
(declare-global db-key-buf)	; list of keys used by apply-db-entry

(setq-default db-entry-buf	"Database Entry")
(setq-default db-key-buf	"Database Key List")

; 
; The hooks defined below are called by mung-db-entry as it processes
; the database entry.  When they are called, the current buffer will
; be the buffer named by db-entry-buf, and will contain the database
; entry.  All the hooks are passed two arguments:  the first is the
; name of the database with which the entry is associated, and the
; second is the key assigned to the entry.  The order in which the
; hooks are called is
; 
; 	db-entry-creation-hook		if the entry is new
; 	db-entry-mung-hook
; 	db-entry-action
; 	db-entry-update-hook		if the entry was modified
;
; db-entry-creation-hook is called if there was no entry for the given key,
; or if there was an entry but it was empty.  It is invoked with the
; current buffer (db-entry-buf) empty.  It can leave anything it wants in
; the buffer; whatever it leaves will be the initial contents of the new
; entry when db-entry-action is called.  If this hook returns zero, the
; entry will not be processed further (hence a new entry will not be created).
;
; db-entry-mung-hook is called by mung-db-entry after it has fetched the
; database entry into db-entry-buf but before it has called db-entry-action.
; This is useful when you want to use the default &&db-entry-recursive-edit
; function and still be able to do something to the entry before editing it.
; If db-entry-mung-hook returns zero, the entry will not be processed
; further, and mung-db-entry will exit.  This is useful for selecting
; certain entries for processing based on some test of their contents.
; 
; db-entry-update-hook is called by mung-db-entry if buffer-is-modified is
; non-zero when db-entry-action returns.  If this hook returns non-zero, the
; entry will be placed in the database, otherwise it will be discarded.
; 

;
; Define default hook functions.  These defaults cause mung-db-entry to
; invoke &&db-entry-recursive-edit on the entry, and always to write the
; edited entry to the database if it was modified.
; 
(defun (db-revert-hooks
	(defun (db-entry-update-hook	(confirm "save changes (y/n)? ")))
	(defun (db-entry-creation-hook
	    (if (confirm (concat "No entry for " 
			     mdkey ".  Create one (y/n)? "))
		(progn
		    (error-occurred
			 (fetch-database-entry
				(arg 1 "Database: ") "TEMPLATE"))
		    1)
		0)
	       ))
	(defun (db-entry-mung-hook	1))
	(defun (db-entry-action
		(&&db-entry-recursive-edit (arg 1) (arg 2))))
       ))
(db-revert-hooks)

;
; open-db makes sure the database file is in the search list for the
; given database.  The file name is found by simply prefixing the database
; directory db-dir to the given name.  The full name so constructed
; is returned.
; 
(defun (open-db	db2  fdb
	(setq db2 (arg 1 "doc database: "))
	(setq fdb  (concat db-dir "/" db2))
	(extend-database-search-list     db2 	fdb)
	fdb
       ))

(defun (close-db
       ))

(defun (confirm
	(c=  (string-to-char (substr (get-tty-string (arg 1)) 1 1))
	 'y')
       ))
; 
; &&db-entry-recursive-edit is the default db-entry-action.  It
; is invoked by mung-db-entry to edit a documentation entry unless somebody
; has redefined db-entry-action.
; 
(defun (&&db-entry-recursive-edit
	(local-bind-to-key "&&db-entry-recursive-edit-abort" "\^G")
	(local-bind-to-key "exit-emacs" "\^X\^S")
	(setq mode-line-format (concat
		"Editing db: " (arg 1) "  key: " (arg 2) "  %*"))
	(recursive-edit)
       ))

(defun (&&db-entry-recursive-edit-abort
	(if (confirm
		"Really abort this recursive edit on this entry (y/n)? ")
	    (progn
		(setq buffer-is-modified 0)
		(message "Aborted.")
		(exit-emacs)
	    ))
       ))

;
; edit-db-entry recursively edits a single database entry.
;
; Inputs:	database name
;		key name
;
(defun (edit-db-entry db key1
	(setq db  (arg 1 "Database: "))
	(setq key1 (arg 2 "Key: "))
	(defun (db-entry-action (&&db-entry-recursive-edit (arg 1) (arg 2))))
	(open-db db)
	(mung-db-entry db key1)
	(novalue)
       ))


; 
; Database entry selection.
; 
; db-list-keys generates a list of keys that are defined in the given
; database, into the buffer db-key-buf.
; 
; db-apropos-keys generates a similar list, but of only those keywords
; which contain a given string.
; 
(defun (db-list-keys	raw-key-lister  entry-selector-filter  lk-db
			lk-full-db
	(setq lk-db (arg 1 "Database: "))
	(setq lk-full-db (open-db lk-db))
	(setq raw-key-lister		(concat "dblist " lk-full-db))
	(setq entry-selector-filter 	"sort")
	(switch-to-buffer db-key-buf)
	(setq needs-checkpointing 0)
	(erase-buffer)
	(insert-string lk-db) (newline)
	(insert-string "----------------") (newline)
	(set-mark)
	(filter-region  (concat raw-key-lister "|" entry-selector-filter))
	(novalue)
       ))

(defun (db-apropos-keys  dak-db dak-kywd
	(setq dak-db (arg 1 "Database: "))
	(setq dak-kywd (arg 2 "Apropos: "))
	(db-list-keys dak-db)
	(switch-to-buffer db-key-buf)
	(filter-region (concat "grep -i """ dak-kywd """"))
	(novalue)
       ))
	
; 
; db-init-key-scan assumes that db-key-buf contains a list of database keys
; such as that generated by db-list-keys, with the database name at the
; beginning followed by a single line of dashes as a separator, followed by
; the names of the keys, one per line.  It opens the database and sets the
; following global variables:
;
;	db-current-full 	to 	the full name of the database,
;					as returned by open-db
;	db-current		to	the base name of the database,
;					as found in db-key-buf
;

(defun (db-init-key-scan	initdb
	(switch-to-buffer db-key-buf)
	(beginning-of-file)
	(error-occurred (re-replace-string "^\n" ""))
	(end-of-file)
	(re-search-reverse "^.")
	(end-of-line)
	(newline)
	(beginning-of-file)
	(set-mark)
	(end-of-line)
	(setq db-current-full (open-db (setq initdb (region-to-string))))
	(beginning-of-file)
	(re-search-forward "^--*\n")
	(setq db-current initdb)
	initdb
))

;
; db-get-next-key returns the database key on the current line in the
; database key buffer.  If its argument is nonzero it steps to the next
; key.
;
(defun (db-get-next-key
	    (switch-to-buffer db-key-buf)
	    (if (eobp)
		0
		(progn	temp
		    (beginning-of-line)
		    (set-mark)
		    (end-of-line)
		    (setq temp (region-to-string))
		    (if (arg 1) (forward-character))
		    (beginning-of-line)
		    temp
		))
))


; 
; mung-db-entry	assumes that the database has been opened (i.e., an
; extend-database-search-list has been done on it) and that db-entry-action
; has been defun'ed as a set of expressions to be executed on the entry.
; If there is no entry for the given key, mung-db-entry calls
; db-entry-creation-hook, which may leave an initial template entry
; in the buffer.  Then db-entry-action is called.  If that function leaves
; buffer-is-modified set, then db-entry-update-hook is called.  If that
; hook returns non-zero, the entry is placed in the database.
; 
(defun (mung-db-entry mdkey   mddb  md-new-entry
    (save-window-excursion   continue
	(setq mddb	 	(arg 1 "Database: "))
	(setq mdkey		(arg 2 "Key: "))
	(switch-to-buffer db-entry-buf)
	(erase-buffer)
	(setq continue 1)
	(if (error-occurred  (fetch-database-entry mddb mdkey))
	    (setq continue (db-entry-creation-hook  mddb  mdkey))
	)
	(if continue
	    (progn 
		(beginning-of-file)
		(if (db-entry-mung-hook mddb mdkey)
		    (progn
			(setq buffer-is-modified 0)
			(db-entry-action mddb mdkey)
			(switch-to-buffer db-entry-buf)
			(if (& buffer-is-modified
			       (db-entry-update-hook mddb mdkey))
			    (put-database-entry mddb mdkey)
			)
		    ))
		(setq buffer-is-modified 0)
	    ))
	)
	(novalue)
))

; 
; apply-to-entries performs the db-entry-action function on all
; the entries that had been listed in the db-key-buf.
; 
(defun (apply-to-entries		 atedb  key
	    (setq atedb (db-init-key-scan))
	    (while (!= (setq key (db-get-next-key 1)) "")
		(mung-db-entry atedb key)
	    )
	(novalue)
       ))


;
; db-edit pops to db-key-buf, opens the database whose entries are
; displayed there, and sets up key bindings to allow you to edit entries.
; The function db-edit-hook is called, if it is defined, with the database
; key list buffer as the current buffer.
;
(declare-global db-current)
(declare-global db-current-full)
(defun (db-edit
	(pop-to-buffer db-key-buf)
	(delete-other-windows)
	(beginning-of-file)
	(db-init-key-scan)
	(local-bind-to-key "mung-next-entry" "\ee")
	(local-bind-to-key "mung-next-entry" "e")
	(local-bind-to-key "previous-line" "p")
	(local-bind-to-key "next-line" " ")
	(local-bind-to-key "next-line" "n")
	(error-occurred (db-edit-hook))
))

(defun (mung-next-entry
    (save-window-excursion
	(pop-to-buffer db-key-buf)
	(mung-db-entry db-current (db-get-next-key 1))
       )))




; 
; make-db-gen-script creates a shell script file that, when executed, will
; generate the given database, with the entries that were listed in the key
; list buffer.
; 
(defun (make-db-gen-script	srcdb	srcdb-full  shname  term ekey
;	(setq srcdb (get-tty-string "Database: "))
;	(db-list-keys srcdb)
	(setq srcdb (db-init-key-scan))
	(setq shname (concat srcdb ".sh"))
	(setq term "_Cretaceous%Ferns!_")
	(switch-to-buffer shname)
	(erase-buffer)

	(while (!= (setq ekey (db-get-next-key)) "")
	    (progn 
	       (insert-string "dbadd " srcdb " """ ekey """<<" """" term """")
		(newline)
		(fetch-database-entry srcdb ekey)
		(end-of-file)
		(newline)
		(insert-string term)
		(newline)
	    )
	)
;	(write-named-file shname)
))
