;----------------------------------------------------------------------------
;NAME
;   rmail -- mail-reading mode
;
;SYNOPSIS
;   rmail	-- calls up this mode
;
;KEY BINDINGS
;   rmail-next-line		next-line		n
;   rmail-previous-line		previous-line		p
;   rmail-search-forward	search-forward		^S
;   rmail-search-reverse	search-reverse		^R
;   rmail-first-message		beginning-of-file	<
;   rmail-last-message		end-of-file		>
;   rmail-find-new-message	next-error		^n
;   rmail-goto-message		goto-line		g
;   rmail-skip						s
;   rmail-previous-page		backward-character	b
;   rmail-next-page		forward-character	f, <space>
;   rmail-delete-message				d
;   rmail-undelete-message				u
;   rmail-append		append-to-file		a
;   rmail-classify					c
;   rmail-find-classified				l
;   rmail-query-exit		illegal-operation	^G
;   exit-emacs						q
;   smail						m
;   rmail-reply						r
;   rmail-wholeheaders					w
;   rmail-help						?
;   shell						!
;   execute-extended-command				:
;
;DESCRIPTION
;   The rmail mode is used for reading mail.  Executing it puts your incoming
;mailbox into a window and enters a special command interpretation loop. It
;understands the following commands, as well as the rebinds indicated above
;(each command listed as rebound in the KEY BINDINGS above is also locally
;bound to its alternate in this mode).
;
;	p	move to the previous message (up-arrow is rebound to this).
;	n	move to the next message (down-arrow is rebound to this).
;
;	^N	move to the first "unseen" message.
;	<	move to the first message.
;	>	move to the last message.
;	^S	search for a given string in a message header.
;       ^R	search backwards for a given string in a message header
;	g	go to message number <n>.
;	s	skip the next <n> messages.
;
;	f	move forward in the current message (left-arrow, space also).
;	b	move backward in the current message (right-arrow also).
;
;	d	delete the current message.
;	u	undelete the last deleted message.
;
;	c	classify current message with a single non-numeric character.
;	l	locate previously classified messages.
;	w	toggle display of junk headers.
;
;	a	append the current message to a file.
;
;	m	send mail.
;	r	reply to the current message.
;	q	quit out of RMail, saving all undeleted messages.
;
;       ?	interactive help
;       !       spawn a shell
;       :	execute extended command      
;
;   It will sometimes be useful to edit other mailboxes than your system
;default in /usr/spool/mail (most notably your blindcopy box). To do this,
;call the function (read-mail) with a mailbox name argument anytime after
;rmail has been loaded. In particular, to clean up your blindcopy box, do
;
;   (read-mail "~/BlindBox")
;
;   There is an (rmail-hook) that is executed just after each time your
;mail is checked.
;
;SEE ALSO
;   smail.ml	-- used to reply to incoming mail
;
;FILES
;   ~/TextMessage		-- default file to append message text to
;   ~/Messages/*		-- where messages are put
;   ~/Messages/Directory	-- where the message list is generated to
;   /usr/spool/mail/$USER	-- user's mailbox
;
;HISTORY
;   16-Oct-84  Eric S. Raymond at Rabbit Software
;	Removed "In-Reply-To:" from the default garbage-headers list.
;   16-Oct-84  Eric S. Raymond at Rabbit Software
;	Fixed bug that was preventing message deletion from working.
;   27-Jul-84  Eric S. Raymond at Rabbit Software
;	Added mail directory cursor wraparound, minor bug fixes.
;   20-Jul-84  Eric S. Raymond at Rabbit Software
;	Dyked out the original code's attempt to read your home directory
;	as a mailbox. Someone was an idiot.
;   4-Jul-84   Eric S. Raymond at Rabbit Software
;	Added this documentation header.
;   The undated past:
;	Modified by Jerry Agin
;	Written by James Gosling
;----------------------------------------------------------------------------

(if (! (is-bound &show-whole-headers))
    (setq-default &show-whole-headers 0))

(defun
    ; The top level mail reader
    (rmail
	(read-mail mailbox-name)
    )

    (read-mail				; read mail from given mailbox
	mailbox
	(setq mailbox (arg 1 "Mailbox: "))
	(if (is-bound &rmail-is-active)
	    (check-mail mailbox)	; leaves user in the directory buffer
	    (do-rmail mailbox)		; this creates the directory buffer
	)
    )

    ; create the rmail buffer and check mail in the given mailbox
    (do-rmail
	entry-name &rmail-is-active
	(message "Please wait while I read your mail file...")
	(sit-for 0)
	(save-window-excursion
	    (pop-to-buffer "rmail-directory")
	    (setq mode-line-format
		(concat "     Mail from message file "
		    (substr mail-directory 1 -1)
		    "      %M   %[%p%]   %<L--More--%>"))
	    (setq needs-checkpointing 0)
	    (setq case-fold-search 1)
	    (setq mode-string "RMail")
	    (check-mail (arg 1))
	    (message "Type ^C to exit rmail; ? for help")
	    (setq read-only 1)
	    (rmail-mark)
	    (recursive-edit)
	    (pop-to-buffer "rmail-directory")
	    (rmail-unmark)
	    (setq read-only 0)
	    (erase-messages)
	    (if buffer-is-modified (write-current-file))
	)
	(novalue)
    )

    ; bring in both old and new mail; takes a mailbox name as argument
    (check-mail
	remember
	(pop-to-buffer "rmail-directory")
	(setq read-only 0)
	(setq remember (+ (dot)))
	(if buffer-is-modified (write-current-file))
	(check-new-mail (arg 1))
	(read-file mail-index-file)
	(beginning-of-file)
	(if (! (eolp)) (insert-character '\n'))
	(setq read-only 1)
	(goto-character remember)
	(position-on-dirent)
	(pick-up-message)
	(error-occurred (rmail-hook))
    )

    ; bring in new mail from a mailbox, takes mailbox name as arg
    (check-new-mail
	(set-mark)
	(fast-filter-region
	    (concat
		emacs-helper-bin-dir "collectmail "
		mail-directory
		" "
		(arg 1)
	    )
	)
	(delete-to-killbuffer)
    )
)

(defun
    (position-on-dirent
	(pop-to-buffer "rmail-directory")
	(if (= (buffer-size) 0)
	    (error-message "You have no new or old mail."))
	(beginning-of-line)
	(if (! (looking-at "^.[^0-9\n]*\\([0-9a-f][0-9a-f]*\\)"))
	    (progn (beginning-of-file)
		   (if (error-occurred (re-search-forward "^ N"))
		       (end-of-file))
		   (rmail-mark)
		   (if (! (looking-at "^.[^0-9\n]*\\([0-9a-f][0-9a-f]*\\)"))
		       (error-message "No messages"))))
	(region-around-match 1)
	(setq entry-name (region-to-string))
	(beginning-of-line)
    )
)

(defun
    (rmail-wholeheaders
	(if &show-whole-headers
	    (progn
		(setq &show-whole-headers 0)
		(message "Headers will be abbreviated.")
	    )
	    (progn
		(setq &show-whole-headers 1)
		(message "Headers will not be abbreviated")
	    )
	)
	(error-occurred
	    (save-excursion 
		(temp-use-buffer "current-message")
		(read-file "/dev/null")
	    )
	)
	(pick-up-message)
    )
)

(if (! (is-bound garbage-headers))
    (setq-default garbage-headers
	(concat
	    "^via:.*\n\\|^mail-from:.*\n\\|^origin:.*\n\\|^received:.*\n\\|"
	    "^message-id:.*\n\\|^remailed-.*\n\\|^return-path:.*\n\\|"
	    "^article-.*\n\\|^debug-trace:.*\n\\|^[ \t]*id .*\n"
    	)
    )
)

(defun
    (pick-up-message
	(position-on-dirent)
	(save-excursion nname
	    (pop-to-buffer "current-message")
	    (set-rmail-mode-line-format)
	    (setq needs-checkpointing 0)
	    (setq nname (expand-file-name (concat mail-directory entry-name)))
	    (if (!= nname (current-file-name))
		(progn
		      (read-file (concat mail-directory entry-name))
		      (beginning-of-file)
		      (if (looking-at "From ")
			  (progn
			      (kill-to-end-of-line)
			      (delete-next-character)
			  ))
		      (setq case-fold-search 1)
		      (if (! &show-whole-headers)
			  (progn (set-mark)
				 (error-occurred
				     (save-restriction
					 (re-search-forward "^$")
					 (narrow-region)
					 (beginning-of-file)
					 (error-occurred
					     (re-replace-string
						 garbage-headers
						 ""))))
				 (setq buffer-is-modified 0))))
		(beginning-of-file))
	)
    )
)

(defun
    (erase-messages			; erase messages marked with D
	(save-excursion
	    (beginning-of-file)
	    (setq case-fold-search 0)
	    (error-occurred (re-replace-string "^.n" "  "))
	    (setq case-fold-search 1)
	    (error-occurred fn
		(while 1
		    (re-search-forward "^.D. \\([0-9a-f]*\\)")
		    (region-around-match 1)
		    (setq fn (concat mail-directory (region-to-string)))
		    (if (unlink-file fn)
		       (progn (message "Couldn't unlink " fn) (sit-for 2))
		    )
		    (beginning-of-line)
		    (set-mark)
		    (end-of-line)
		    (forward-character)
		    (erase-region)
		)
	    )
	)
    )

    (rmail-com
	(argc) (rmail) (exit-emacs))

    (rmail-next-page
	(save-excursion
	    (pop-to-buffer "current-message")
	    (next-page)
	    (set-rmail-mode-line-format)
	))

    (rmail-previous-page
	(save-excursion
	    (pop-to-buffer "current-message")
	    (previous-page)
	    (set-rmail-mode-line-format)
	))

    (set-rmail-mode-line-format
	    (setq mode-line-format
		"     Current message       %[%p%]   %<L--More--%>"))

    (rmail-previous-line
	(beginning-of-line) (backward-character)
	(if (bobp)
	    (end-of-file)
	))

    (next-line-wrap
	(end-of-line) (forward-character)
	(if (eobp)
	    (beginning-of-file)
	))

    (rmail-next-message
	(save-excursion (rmail-unmark))
	(provide-prefix-argument prefix-argument (next-line-wrap))
	(rmail-mark)
    )

    (rmail-previous-message
	(save-excursion (rmail-unmark))
	(provide-prefix-argument prefix-argument (rmail-previous-line))
	(rmail-mark)
    )

    (rmail-delete-message
	(beginning-of-line)
	(setq read-only 0)
	(error-occurred
	    (prefix-argument-loop
		(forward-character)
		(delete-next-character)
		(insert-character 'D')
		(beginning-of-line)
		(next-line)
	    )
	    (previous-line)
	)
	(setq read-only 1)
	(rmail-check-mark)
    )
    (rmail-undelete-message
	(beginning-of-line)
	(setq read-only 0)
	(error-occurred
	    (prefix-argument-loop
		(forward-character)
		(delete-next-character)
		(insert-character ' ')
		(beginning-of-line)
		(next-line)
	    )
	    (previous-line)
	)
	(setq read-only 1)
	(rmail-check-mark)
    )
    (rmail-classify char
	(rmail-check-mark)
	(message "Character: ")
	(setq char (get-tty-character))
	(if (< char ' ')
	    (error-message "Control characters not allowed!")
	    (if (& (>= char '0') (<= char '9'))
		(error-message "Numeric identifiers not allowed!")
		(progn
		    (setq read-only 0)
		    (forward-character)
		    (forward-character)
		    (delete-next-character)
		    (insert-character char)
		    (beginning-of-line)
		    (setq read-only 1)
		)
	    )
	)
    )
    (rmail-find-new-message 
		start-pos
	(rmail-unmark)
	(setq start-pos (dot))
	(setq case-fold-search 0)
	(if (error-occurred 
		(progn 
		       (beginning-of-file)
		       (re-search-forward "^ N")
		)
	    )
	    (progn
		  (goto-character start-pos)
		  (provide-prefix-argument prefix-argument
		      (next-line))
		  (if (eobp) (progn (previous-line)
				    (message "You're at the last message")))
	    )
	)
	(rmail-mark)
    )
    (rmail-find-classified
	char search-string
	(message "Character: ")
	(setq char (get-tty-character))
	(setq search-string (concat "^.." (char-to-string char)))
	(next-line)
	(setq case-fold-search 0)
	(if (error-occurred (re-search-forward search-string))
	    (progn
		  (previous-line)
		  (error-message "Can't find it!")
	    )
	    (progn
		  (save-excursion (rmail-unmark))
		  (rmail-mark)
	    )
	)
    )
)

(autoload "&info" "info.ml")

(defun
    (rmail-help
	(&info "emacs" "rmail")))

(autoload "rmail-reply" "smail.ml")
(autoload "smail" "smail.ml")

(defun

    ; remove the > mark on the current line
    (rmail-unmark
	(error-occurred
	    (position-on-dirent)
	    (setq read-only 0)
	    (delete-next-character)
	    (insert-character ' ')
	    (setq read-only 1)
	)
    )

    ; update the status mark on the current line
    (rmail-check-mark
	(if (! (looking-at "^>[^0-9\n]*\\([0-9a-f][0-9a-f]*\\)"))
	    (progn
		  (save-excursion (rmail-unmark))
		  (rmail-mark)
	    )
	)
    )

    ; set the status mark on the current line
    (rmail-mark
	  (if (error-occurred
		(beginning-of-line)
		(if (eobp)
		    (re-search-reverse "^.")
		    (progn
			(re-search-forward "^.")
			(beginning-of-line)))
		(setq read-only 0)
		(delete-next-character)
		(insert-character '>')
		(if (= (following-char) 'N')
		    (progn
			  (delete-next-character)
			  (insert-character 'n')
		    )
		)
		(setq read-only 1)
		(beginning-of-line)
		(pick-up-message)
	      )
	    (message "No messages")
	  )
    )
)

(declare-global &rmail-search-string)

(defun
    (rmail-search-forward ss
	(setq case-fold-search 1)
	(setq ss (get-tty-string "Search forward: "))
	(if (= ss "")
	    (setq ss &rmail-search-string)
	    (setq &rmail-search-string ss))
	(next-line)
	(if (error-occurred (re-search-forward ss))
	    (progn
		  (previous-line)
		  (error-message "Can't find it!")
	    )
	    (progn
		  (save-excursion (rmail-unmark))
		  (rmail-mark)
	    )
	)
    )
    (rmail-search-reverse ss
	(setq case-fold-search 1)
	(setq ss (get-tty-string "Search reverse: "))
	(if (= ss "")
	    (setq ss &rmail-search-string)
	    (setq &rmail-search-string ss))
	(if (error-occurred (re-search-reverse ss))
	    (error-message "Can't find it!")
	    (progn
		  (save-excursion (rmail-unmark))
		  (rmail-mark)
	    )
	)
    )
    (rmail-goto-message n
	(if prefix-argument-provided
	    (setq n prefix-argument)
	    (setq n (arg 1 "Goto message: "))
	)
	(rmail-unmark)
	(beginning-of-file)
	(provide-prefix-argument n (next-line))
	(rmail-mark)
    )
    (rmail-first-message
	(rmail-unmark)
	(beginning-of-file)
	(rmail-mark)
    )
    (rmail-last-message
	(rmail-unmark)
	(end-of-file)
	(rmail-mark)
    )
    (rmail-skip n
	(setq n (get-tty-string "Skip messages: "))
	(save-excursion (rmail-unmark))
	(provide-prefix-argument n (next-line))
	(rmail-mark)
    )
)
    
(declare-global rmail-default-log)
(setq rmail-default-log "~/TextMessage")

(defun
    (rmail-append file
	(setq file (get-tty-file ": append-message-to-file ["
				       rmail-default-log "]  "))
	(if (= file "") (setq file rmail-default-log))
	(save-excursion 
	    (temp-use-buffer "current-message")
	    (append-to-file file))
	(setq rmail-default-log file)
    )
    (rmail-query-exit
	   (if (!= "y" (substr (get-tty-string
			   "Do you really want to abort mail reading? ")
			       1 1))
	       (error-message "Oh very well then...goodbye!"))
	   (exit-emacs)
       )
)

(if (! (is-bound mail-directory))
    (setq-default mail-directory (concat (getenv "HOME") "/Messages/")))
(if (! (is-bound mail-index-file))
    (setq-default mail-index-file (concat mail-directory "Directory")))
(if (! (is-bound mailbox-name))
    (setq-default mailbox-name (expand-file-name "/usr/spool/mail/$USER")))

(defun
    (bind-and-rebind
	(local-rebind (arg 1) (arg 2) (arg 3))
	(local-bind-to-key (arg 1) (arg 3))
    )
)

(save-excursion i
    (temp-use-buffer "rmail-directory")
    (define-keymap "rmail-commands")
    (use-local-map "rmail-commands")

    ; elementary commands to change current message
    (bind-and-rebind "rmail-next-message" "next-line" "n")
    (bind-and-rebind "rmail-previous-message" "previous-line" "p")

    ; smart message-move commands
    (bind-and-rebind "rmail-first-message" "beginning-of-file" "<")
    (bind-and-rebind "rmail-last-message" "end-of-file" ">")

    (bind-and-rebind "rmail-find-new-message" "next-error" "\^n")
    (bind-and-rebind "rmail-goto-message" "goto-line" "g")
    (local-bind-to-key "rmail-skip" "s")

    (bind-and-rebind "rmail-search-forward" "search-forward" "\^s")
    (bind-and-rebind "rmail-search-reverse" "search-reverse" "\^r")

    ; elementary commands to move about within a message
    (bind-and-rebind "rmail-previous-page" "backward-character" "b")
    (bind-and-rebind "rmail-next-page" "forward-character" "f")
    (local-bind-to-key "rmail-next-page" " ")    

    ; delete and undelete commands
    (local-bind-to-key "rmail-delete-message" "d")
    (local-bind-to-key "rmail-undelete-message" "u")

    (bind-and-rebind "rmail-append" "append-to-file" "a")
    (local-bind-to-key "rmail-append" "a")
    
    (local-bind-to-key "rmail-classify" "c")
    (local-bind-to-key "rmail-find-classified" "l")

    (bind-and-rebind "rmail-query-exit" "illegal-operation" "\^G")
    (local-bind-to-key "exit-emacs" "q")

    (local-bind-to-key "smail" "m")
    (local-bind-to-key "rmail-reply" "r")

    (local-bind-to-key "rmail-wholeheaders" "w")
    (local-bind-to-key "rmail-help" "?")

    (local-bind-to-key "shell" "!")
    (local-bind-to-key "execute-extended-command" ":")
)

(if (! (is-bound &smail-is-loaded))	; This hack is necessary because
    (defun				;  the autoload function will not
	  (smail			;  allow redeclaration
		(load  "smail.ml")
		(smail)
	  )
    )
)

; rmail.ml ends here ---------------------------------------------------------
