;---------------------------------------------------------------------------
;NAME
;   smail.ml -- send mail from Emacs
;
;SYNOPSIS
;   smail	-- calls up this mode
;
;KEY BINDINGS
;   See below
;
;DESCRIPTION
;   The "smail" mode is used for sending mail.  It places you in a buffer for
;constructing the message and locally defines a few commands:
;
;  ^X^S	send the mail -- if all went well the window will disappear,
;	otherwise a message indicating which addresses failed will appear
;	at the bottom of the acreen.  Unfortunately, the way the mailers on
;	Unix work, the message will have been sent to those addresses which
;	succeeded and not to the others, so you have to delete some
;	addresses and fix up the others before you resend the message.
;
;  ^Xt	sets you up to recursive-edit the To: field of the message (use ^C or
;	something else bound to exit-emacs to leave). The abbreviations in
;	emacs-helper-bin-dir/RMailAbbrevs are enabled while you're here.
;
;  ^Xc	positions you in the Cc: field of the message, creating it if it
;	doesn't already exist. Then it does a recursive-edit with abbrevs
;	enabled as for the To: field.
;
;  ^Xs	positions you in the Subject: field of the message for a recursive
; 	edit. Abbreviations are not enabled.
;
;  ^Xa	positions you to the end of the body of the message, ready to
; 	append more text.
;
;   The code then does the equivalent of an ^Xt followed by ^Xs, followed
;by ^Xa, so you're walked through filling out the To:-field and then the
;Subject:-field and then to the message text.
;
;   The format of your 'from' line is determined by the &domain-name
;variable, which defaults to "" to call for your full name (as determined
;from the gecos field of your password entry. The values "UUCP" and "ARPA"
;give formats suitable for those nets. No "Message-id" field is generated
;for the message unless &domain-name is set to "ARPA".
;   Any other domain name than UUCP or ARPA will simply be appended to the
;originating name following a dot. The following table shows how user J. Fred
;Muggs aka 'muggs' at system 'foo' will see his 'From:' line for various
;domain-name choices:
;	""	From:	J. Fred Muggs <muggs>
;	"UUCP"	From:	J. Fred Muggs <foo!muggs>
;	"ARPA"	From:	J. Fred Muggs <muggs@foo.ARPA>
;	"BLOG"	From:	J. Fred Muggs <muggs@foo.BLOG>
;
;   The abbrev facility is used for mail address expansion; the file
;emacs-helper-bin-dir/RMailAbbrevs should contain abbrev definitions to
;expand login names to their proper mail addresses.
;   There is an 'smail-hook' call that can be used for customization that
;is called at the end of the mode initialization.
;
;SEE ALSO
;   /bin/mail(1) -- 4.2bsd and V7/System III/System V mail sending utility
;   delivermail(8) -- 4.1bsd utility used to send mail
;   rmail.ml -- the mail-reader (calls some functions in this package)
;   finger.ml -- used to get user's actual full name
;   info.ml -- used to provide interactive help
;
;FILES
;   /usr/local/lib/emacs/RMailAbbrevs -- mail group abbreviations
;   ~/BlindBox -- blindcopies of your messages are appended to this
;   /usr/spool/mail/~ -- where we look for mail
;
;CAVEAT
;   You must set the variable mail-sender to the pathname of the
;   utility used to send mail on your system, followed by any
;   flags it needs.  This string will be typically be one of
; 	V7, SIII, SV	"/bin/mail"
; 	BSD4.1		"/etc/delivermail -i -s -p"
;	BSD4.2		"/etc/sendmail"
;
;HISTORY
;   17-Oct-84   Eric S. Raymond at Rabbit Software
;	Fixed losing check-mail calls, set code up to emit RFC 822 format.
;   12-July-84  Eric S. Raymond at Rabbit Software
;	Changed "To: " format to include both finger and login names.
;   10-July-84  Eric S. Raymond at Rabbit Software
;	Added documentation header. Fixed the code to know about finger's
;	(ab)use of the gecos field (i.e. gathering of chars for the username
;	stops on the first comma if there is one). Added the network-name
;	variable.
;---------------------------------------------------------------------------


; The following 2 lines should be changed when you install Emacs
	(setq-default TimeZone "EST")
	(setq-default mail-sender "/bin/mail")

(if (! (is-bound &domain-name))
    (setq-default &domain-name ""))	; By default, local mail format

(if (! (is-bound &check-mail-frequently))
    (setq-default &check-mail-frequently 1))

(if (! (is-bound blindbox-name))
    (setq-default blindbox-name (expand-file-name "~/BlindBox")))

(if (! (is-bound mailbox-name))
    (setq-default mailbox-name (expand-file-name "/usr/spool/mail/$USER")))

(declare-global &smail-is-loaded)
(declare-global mail-do-send)

(autoload "finger-name" "finger.ml")	; So we know about user names
(autoload "&info" "info.ml")		; So we can get help

;---------------------------------------------------------------------------
;rmail-help
;   Get interactive help about the mail system.
;---------------------------------------------------------------------------
(defun
    (rmail-help
	(&info "emacs" "rmail")
    )
)

;---------------------------------------------------------------------------
;smail-com
;   Assuming there is an (autoload "smail-com" "smail.ml") in your profile,
;typing 'emacs -esmail-com' will execute an EMACS specialized to be a mail
;reader; its argument list will be interpreted as a list of addressees. 
;---------------------------------------------------------------------------
(defun
    (smail-com i
	(declare-global exit-when-through)
	(pop-to-buffer "send-mail")
	(setq needs-checkpointing 0)
	(setq case-fold-search 1)
	(erase-buffer)
	(if (! (is-bound read-mail-abbrevs))
	    (progn (declare-global read-mail-abbrevs)
		(quietly-read-abbrev-file
			(concat emacs-helper-bin-dir "RMailAbbrevs"))
	    )
	)
	(use-abbrev-table "RMail")
	(setq i 1)
	(newline)
	(while (> (argc) i)
	    (if (!= (substr (argv i) 1 1) "-")
		(progn
		    (insert-string (argv i))
		    (if (> (argc) i) (insert-character ','))
		)
	    )
	    (setq i (+ i 1))
	)
	(newline)
	(accept-mail "" "" "")
	(exit-emacs)
    )
)

;---------------------------------------------------------------------------
;rmail-reply
;   Reply to the message currently being read. This MUST be called only from
;an active rmail.
;---------------------------------------------------------------------------
(defun
    (rmail-reply
	subject dest in-reply-to
	(if (is-bound &send-mail-is-active)
	    (error-message
		"You're already composing one message, finish it first."))

	(save-window-excursion
	    (pop-to-buffer "current-message")
	    (setq case-fold-search 1)
	    (beginning-of-file)
	    (search-forward "\n\n")		; search for message text
	    (set-mark)
	    (beginning-of-file)
	    (narrow-region)			; look only at headers

	    ; set the subject field
	    (setq subject "")
	    (error-occurred
		(re-search-forward "^Subject:[ \t]*\\(.*\\)")
		(region-around-match 1)
		(setq subject (region-to-string))
;		(if (!= (substr subject 1 3) "Re:")
;		    (setq subject (concat "Re: " subject))
;		)
	    )

	    ; set the destination field
	    (setq dest "")
	    (beginning-of-file)
	    (error-occurred
		(if (error-occurred (re-search-forward
				       "^reply-to:[ \t]*\\(.*\\)"))
		    (re-search-forward "^from:[ \t]*\\(.*\\)"))
		(region-around-match 1)
		(setq dest (region-to-string))
	    )

	    ; set the in-reply-to field
	    (beginning-of-file)
	    (setq in-reply-to "")
	    (error-occurred
		edest
		(save-excursion 
		    (temp-use-buffer "Scratch Stuff")
		    (setq needs-checkpointing 0)
		    (erase-buffer)
		    (insert-string dest)
		    (set-mark)
		    (beginning-of-file)
		    (error-occurred 
			(re-replace-string
			    "  *at  *[^,\n]*\\| *@ *[^,\n]*\\| *([^)\n]*)\\| *<[^>\n]*>"
			    ""))
		    (setq edest (region-to-string)))
		(re-search-forward "^date:[ \t]*")
		(set-mark)
		(end-of-line)
		(setq in-reply-to (concat
				    "In-Reply-To: "
				    edest "'s message of "
				    (region-to-string)
				    "\n"))
	    )

	    (widen-region)
	    (pop-to-buffer "send-mail")
	    (setq case-fold-search 1)
	    (erase-buffer)
	    (accept-mail dest subject in-reply-to)
	)
	(position-on-dirent)
	(if (looking-at "^ [Nn ]")
	    (progn
		  (forward-character)
		  (setq read-only 0)
		  (delete-next-character)
		  (insert-character 'A')
		  (setq read-only 1)
		  (beginning-of-line)))
	(if &check-mail-frequently (check-mail mailbox-name))
    )

;---------------------------------------------------------------------------
;mail-insert
;   Insert one of the messages in the current read list in the message being
;composed. This must be called from an active rmail.
;---------------------------------------------------------------------------
    (mail-insert
	(mail-append)
	(save-excursion
	    (insert-string "\n_________________________________\n")
	    (yank-buffer "current-message"))
	(re-replace-string "^From \\|^Date:" ">&"))
    
;---------------------------------------------------------------------------
;mail-noblind-exit
;   Exit and send the composed message without sending a blindcopy.
;---------------------------------------------------------------------------
    (mail-noblind-exit
	(setq mail-append-blind 0)
	(exit-emacs))
    
;---------------------------------------------------------------------------
;send-mail
;   This is the major entry point. It initiates mail sending.
;---------------------------------------------------------------------------
    (smail
	(if (is-bound &send-mail-is-active)
	    (error-message
		"You're already composing one message, finish it first."))
	(save-window-excursion
	    (pop-to-buffer "send-mail")
	    (setq case-fold-search 1)
	    (erase-buffer)
	    (accept-mail "" "" "")
	)
	(if (is-bound &rmail-is-active)
	    (if &check-mail-frequently (check-mail mailbox-name))
	)
	(novalue)
    )

;-----------------------------------------------------------------------------
;accept-mail
;   Format the send-mail buffer to accept a message. This creates the minimum
;format assumed by Internet RFC 822 (Date, From, Subject and To fields) plus
;a Message-Id field if the domain-name is "ARPA".
;   This function accepts three arguments: a destination string, a subject
;string, and an extra-fields string. The first two arguments become the
;subject and destination fields of the message; either of these that is empty
;(i.e. "") will be queried for. The third is for extra fields (such as an
;"In-Reply-To:" and/or "Cc:" field) which are not queried for.
;-----------------------------------------------------------------------------
    (accept-mail		; set things up in a mail sending buffer.
	mail-append-blind time
	(pop-to-buffer "send-mail")
	(setq mode-line-format
	      "  Message composition      %M   %[%p%]   %<L--More--%>")
	(beginning-of-file)

	(setq time (current-time))
	(insert-string
	    "Date: "
	    (substr time 1 3) " "		; Day
	    (if (= (substr time 9 1) " ")	; Date
		(substr time 10 1)		;   (one-digit case)
		(substr time 9 2))		;   (two-digit case)
	    " "
	    (substr time 5 3) " "		; Month
	    (substr time -4 4) " "		; Year
	    (substr time 12 8) " "		; Time
	    TimeZone
	    "\n"
	)

	(insert-string
	    "From: " (finger-name) " <"
	    (if
	    	(= &domain-name "")
		    (concat (users-login-name))
		(= &domain-name "UUCP")
		    (concat (system-name) "!" (users-login-name))
	    	1
		    (concat
			(users-login-name)
			"@" (system-name)
			"." &domain-name
		    )
	    )
	    ">\n"
	)

	(insert-string "Subject: \n")
	(insert-string "To: \n")

	(if (= &domain-name "ARPA")
	    (insert-string "Message-Id: <"
		(current-numeric-time) "/"
		(users-login-name) "@" (system-name) ">\n"))

	(insert-string (arg 3))		; In-Reply-to field (if any)

	(end-of-file)
	(newline)
	(backward-character)
	(if (! (bolp)) (newline))
	(setq mode-string "SMail")
	(setq mail-append-blind 1)
	(if (! (is-bound read-mail-abbrevs))
	    (progn (declare-global read-mail-abbrevs)
		   (quietly-read-abbrev-file
		       (concat emacs-helper-bin-dir "RMailAbbrevs"))
	    )
	)
	(use-abbrev-table "RMail")
	(beginning-of-file)

	(setq right-margin 72)	
	(local-bind-to-key "rmail-help" "\^X?")
	(local-bind-to-key "mail-to" "\^Xt")
	(local-bind-to-key "mail-append" "\^Xa")
	(local-bind-to-key "mail-cc" "\^Xc")
	(local-bind-to-key "mail-subject" "\^Xs")
	(local-bind-to-key "mail-fetch-field" "\^Xf")
	(local-bind-to-key "mail-insert" "\^Xi")
	(local-bind-to-key "mail-noblind-exit" "\^X\^C")
	(local-bind-to-key "exit-emacs" "\^X\^S")
	(local-bind-to-key "exit-emacs" "\^X\^F")
	(local-bind-to-key "mail-abort-send" "\^G")
	(local-bind-to-key "mail-abort-send" "\^X\^A")
	(local-bind-to-key "justify-paragraph" "\ej")
	(message "Type ^C to send the message; ^X^A to abort; ^X? for help.")
	(error-occurred (smail-hook))
	
	(mail-to (arg 1))		; fill in the To field
	(mail-subject (arg 2))		; fill in the Subject field

	(mail-append)			; accept a message body

	(setq mail-do-send 1)
	(while mail-do-send
	       (save-excursion
		   (recursive-edit)
		   (if mail-do-send
		       (progn
			     (message "Sending...")
			     (sit-for 0)
			     (send-mail)
			)
		   )
	       )
	)
    )

;---------------------------------------------------------------------------
;mail-abort-send
;   Exit send-mail mode without sending the composed message.
;---------------------------------------------------------------------------
    (mail-abort-send
	(if (!= "y" (substr (get-tty-string
				"Do you really want to abort the message? ")
			    1 1)))
	(setq mail-do-send 0)
	(exit-emacs))
    
;---------------------------------------------------------------------------
;mail-to
;   Recursive-edit the addressees field of the current messages. Only here
;are abbreviations enabled from RMailAbbrevs.
;---------------------------------------------------------------------------
    (mail-to
	(beginning-of-file)
	(re-search-forward "^to:.*")
	(setq abbrev-mode 1)
	(if (| (interactive) (= (arg 1 "To: ") ""))
	   (save-excursion (recursive-edit))
	   (insert-string (arg 1))
	)
	(setq abbrev-mode 0)
    )
    
;---------------------------------------------------------------------------
;mail-subject
;   Recursive-edit the subject field of the current message.
;---------------------------------------------------------------------------
    (mail-subject
	(beginning-of-file)
	(re-search-forward "^subject:.*")
	(if (| (interactive) (= (arg 1 "Subject: ") ""))
	   (save-excursion (recursive-edit))
	   (insert-string (arg 1))
	)
    )
    
;---------------------------------------------------------------------------
;mail-fetch-field
;   Fetch a field from the current message header, insert it at dot.
;---------------------------------------------------------------------------
    (mail-fetch-field
	field-name field-contents
	(setq field-name (get-tty-string ": field name "))
	(save-excursion
	    (temp-use-buffer "current-message")
	    (beginning-of-file)
	    (setq case-fold-search 1)
	    (re-search-forward (concat "^" field-name "[^\n:]*:[ \t]*"))
	    (mail-extend-field)
	    (setq field-contents (region-to-string))
	)
	(insert-string field-contents)
    )
    
;---------------------------------------------------------------------------
;mail-append
;   Move to the end of the message.
;---------------------------------------------------------------------------
    (mail-append
	(end-of-file)
	(setq left-margin 1)
    )
    
;---------------------------------------------------------------------------
;mail-cc
;   Recursive-edit a carbon-copy list. Create one following the "To:" if it
;doesn't exist. Abbreviations are enabled.
;---------------------------------------------------------------------------
    (mail-cc
	(beginning-of-file)
	(if (error-occurred (re-search-forward "^cc:.*"))
	    (progn (re-search-forward "\n\n\\|^to:.*\n.")
		   (backward-character)
		   (backward-character)
		   (while (looking-at "^[ \t]")
			  (progn (beginning-of-line)
				 (next-line)))
		   (insert-string "\nCc: ")
	    )
	    (save-excursion
		   (setq abbrev-mode 1)
		   (recursive-edit)
		   (setq abbrev-mode 0)
	    )
	)
    )
    
;---------------------------------------------------------------------------
;mail-extend-field
;   If one is currently in a multi-line field, this will go to the end of it.
;Multi-line fields continuations are recognized by the fact that each line
;in them begins with a space or tab.
;---------------------------------------------------------------------------
    (mail-extend-field
	(set-mark)
	(while (progn (end-of-line)
		      (looking-at "\n[ \t]"))
	       (forward-character))
    )
    
;---------------------------------------------------------------------------
;send-mail
;   Send the current message to the addressees.
;---------------------------------------------------------------------------
    (send-mail addresses
	(beginning-of-file)
	(if (error-occurred (re-search-forward "^$"))	; find the text start
	    (end-of-file))
	(set-mark)
	(beginning-of-file)
	(narrow-region)
	(setq addresses "")
	(error-occurred
	    (re-search-forward "^to:[ \t]*")
	    (mail-extend-field)
	    (setq addresses (region-to-string)))
	(beginning-of-file)
	(error-occurred
	    (re-search-forward "^cc:[ \t]*")
	    (mail-extend-field)
	    (setq addresses (concat addresses "," (region-to-string))))
	(widen-region)
	(message "sending...")
	(save-excursion
	    (temp-use-buffer "Scratch Stuff")
	    (setq needs-checkpointing 0)
	    (erase-buffer)
	    (insert-string addresses)
	    (beginning-of-file)
	    (error-occurred (re-replace-string "[\t\n]" " "))
	    (error-occurred (re-replace-string " *([^(]*)" ""))
	    (error-occurred (re-replace-string "[^,]*<\\([^>,]*\\)>" "\\1"))
	    (error-occurred (re-replace-string "  *at  *\\|  *@ *\\|@  *" "@"))
	    (error-occurred (re-replace-string "  *" "."))
	    (error-occurred (re-replace-string "\\.*,\\.*" " "))
	    (if (= &domain-name "UUCP")
	    	(error-occurred (replace-string "!" "\\!")))
	    (error-occurred (re-replace-string "^  *\\|[ .][ .]*$" ""))
	    (set-mark)
	    (end-of-file)
	    (setq addresses (region-to-string))
	)
	(end-of-file)
	(if (! (bolp)) (newline))
	(beginning-of-file)
	(if (& mail-append-blind (!= blindbox-name ""))
	    (progn
		  (insert-string "\^C\n")
		  (append-to-file blindbox-name)
		  (delete-previous-word)))
	(set-mark)
	(end-of-file)
	(fast-filter-region (concat mail-sender " " addresses))
	(if (= (buffer-size) 0)
	    (progn
		  (setq mail-do-send 0)
		  (message (concat "Sent to " addresses)))
	    (progn no-go
		   (beginning-of-file)
		   (set-mark)
		   (end-of-file)
		   (copy-region-to-buffer "Delivery-errors")
		   (beginning-of-file)
		   (error-occurred (re-replace-string "\n\n* *" "; "))
		   (end-of-line)
		   (setq no-go (region-to-string))
		   (erase-buffer)
		   (yank-from-killbuffer)
		   (beginning-of-file)
		   (error-occurred 
		       (re-search-forward "^to:[^>\n]*"))
		   (message no-go))
	)
    )
)

; smail.ml ends here -------------------------------------------------------
