;---------------------------------------------------------------------------
; wyse.ml	support package for wyse 50, 100 and 300 terminals
;
;SYNOPSIS
;   function-key		-- return keysequence value of fn key
;   shifted-function-key	-- return keysequence value of shifted fn key
;   set-function-key		-- reprogram function key
;   set-shifted-function-key	-- reprogram shifted function key
;
;KEY BINDINGS
;   See below
;
;DESCRIPTION
;   This package defines function-key setting functions, key mappings and mode
;hooks. Wyse100 conventions are assumed unless explicitly stated otherwise.
;
;   The following corrects several lies in wyse100 ref man, table 10, pg T7.
;   * RESET (shifted or not) sends nothing.
;   * SCRL^ sends ESCv
;   * SCRLv sends ESCw
;   * HOME shifted sends ESC{
;
;   And here are some corrections for lies in the Wyse50 manual:
;   * The PAGE key shifted and unshifted values are swapped.
;
;   Note the following differences in the Wyse 50 from the 100 and 300:
;   * PAGE/shift-PAGE goes from ESC-h/ESC-K to ESC-K/ESC-J (see above)
;   * The shifted LINE ERASE and PAGE ERASE keys are gone. 
;   * Unshifted LINE ERASE becomes unshifted CLR (CLR LINE).
;   * Unshifted PAGE ERASE becomes shifted CLR (CLR PAGE).
;   * INSERT LINE is shifted INS (INS LINE)
;   * INSERT CHAR is unshifted INS (INS CHAR)
;   * SEND MSG becomes SEND; PRINT is unusable due to local action.
;   * SCRL UP and SCRL DOWN are gone.
;
;BUGS
;   The Wyse terminals have the following gross, ugly, and painful misfeatures:
;   * Down arrow is a linefeed.
;   * Left arrow is a backspace.
;   * Right arrow is a formfeed.
;   * Highlights take up a screen space.
;   The designer(s) clearly deserve to be flayed and boiled in oil.
;
;NOTE
;   In case your site doesn't have one, here are the termcap entries we
;use:
;
;W1|Wyse100|Wyse 100:\
;	:bs:al=\EE:co#80:li#24:am:\
;	:bt=\EI:cd=\EY:cl=\E+:cm=\E=%+ %+ :dl=\ER:\
;	:ce=\ET:\
;	:dc=\EW:ic=\EQ:\
;	:so=\E):se=\E(:\
;	:up=^k:kb=^H:do=^J:nl=^J:
;W5|Wyse50|Wyse 50:\
;	:ti=\E`6:te=\E(\EA11:\
;	:tc=W1:
;
;   Note that te sets the soft-key label line to nondisplay but doesn't
;clear the legends. It also turns off highlight mode.
;
;HISTORY
;   18-July-84  Eric S. Raymond
;	Reorganized. Now has similar entry points to the other terminal-
;	support packages.
;   04-July-84  Eric S. Raymond and Tom Donohue at Rabbit Software
;	Released.
;---------------------------------------------------------------------------

; First, a function to simplify things for keys that differ on the 50

;----------------------------------------------------------------------------
; onwyse:
; 	return Wyse 100 or Wyse 50 version of key depending on $TERM
;	First arg is Wyse100/300 alternative, second is Wyse 50
;----------------------------------------------------------------------------
(defun
	(onwyse
		(if (= (getenv "TERM") "W5") (arg 2) (arg 1))
	)
)

;****************************************************************************
;
; Functions for defining wyse soft-key label display areas.

;----------------------------------------------------------------------------
; function-key
;	return the default key-sequence value of a numbered function key
;	arg 1 is a function key number
; 	arg 2 is optional and may be a string, up to 8 chars per soft-key.
;----------------------------------------------------------------------------
(defun
    (function-key
	(if (<= (arg 1 ) 8)
	    (send-string-to-terminal
		(concat "\ez" (- (arg 1) 1) (arg 2) "\r")
	    )
	)
	(concat "\^A" (char-to-string (+ (arg 1) 63)) "\r")
    )
)

;----------------------------------------------------------------------------
; shifted-function-key
;	return the default key-sequence value of a shifted function key
;	arg 1 is a function key number
; 	arg 2 is optional and may be a string, up to 8 chars per soft-key.
;----------------------------------------------------------------------------
(defun
    (shifted-function-key
	(if (<= (arg 1 ) 8)
	    (send-string-to-terminal
		(concat "\ez" (- (arg 1) 1) (arg 2) "\r")
	    )
	)
	(concat "\^A" (char-to-string (+ (arg 1) 95)) "\r")
    )
)

;----------------------------------------------------------------------------
; set-function-key
;	program the key-sequence value of a numbered function key
;	arg 1 is a function key number
; 	arg 2 is a string, up to 8 chars per function-key.
;	this function returns the new value
;----------------------------------------------------------------------------
(defun
    (set-function-key
	(send-string-to-terminal
	    (concat "\ez" (+ (arg 1) 63) (arg 2) "\0177")
	)
	(arg 2)
    )
)

;----------------------------------------------------------------------------
; set-shifted-function-key
;	program the key-sequence value of a shifted numbered function key
;	arg 1 is a function key number
; 	arg 2 is a string, up to 8 chars per function-key.
;	this function returns the new value
;----------------------------------------------------------------------------
(defun
    (set-shifted-function-key
	(send-string-to-terminal
	    (concat "\ez" (+ (arg 1) 95) (arg 2) "\0177")
	)
	(arg 2)
    )
)

;****************************************************************************
;
; define wyse keymap
; 
(progn i
	(setq i ' ')
	(while (< i 0177)
		(bind-to-key "self-insert" i)
		(setq i (+ i 1))))

;----------------------------------------------------------------------------
; labeled motion cmds					;  wyse key
; 
(bind-to-key "self-insert" "\t")			; TAB
(bind-to-key "newline" (onwyse "\r" "\e\r"))		; RETURN
(bind-to-key "newline-and-indent" (onwyse "\^_" "\r"))	; NEW LINE
; 
(bind-to-key "forward-character" "\^L")			; > arrow
(bind-to-key "backward-character" "\b")			; < arrow, BACKSPACE
(bind-to-key "next-line" "\n")				; v arrow
(bind-to-key "previous-line" "\^K")			; ^ arrow
; 
(bind-to-key "scroll-one-line-up" "\ev")		; SCRL^
(bind-to-key "scroll-one-line-down" "\ew")		; SCRLv
;
(bind-to-key "line-to-top-of-window" "\e@")		; PRINT
(bind-to-key "redraw-display" "\eP")			; PRINT shifted
;
(bind-to-key "beginning-of-file" "\^^")			; HOME
(bind-to-key "end-of-file" "\e\^^")			; ESC-HOME

(bind-to-key "next-page" (onwyse "\eh" "\eK"))		; PAGE
(bind-to-key "previous-page" (onwyse "\eK" "\eJ"))	; PAGE shifted

;----------------------------------------------------------------------------
; deletes
; 
(bind-to-key "delete-next-character" (onwyse "\eQ" "\eW"))	; INS/DEL CHAR
(bind-to-key "delete-previous-character" "\177")	; RUBOUT
(bind-to-key "kill-to-end-of-line" "\eT")		; LINE ERASE / CLR LINE
(bind-to-key "delete-to-killbuffer" "\eY")		; PAGE ERASE / CLR SCRN
(bind-to-key "delete-buffer" "\ey")			; PAGE ERASE shifted

;----------------------------------------------------------------------------
; inserts
; 
(bind-to-key "newline-and-backup" "\eE")		; LINE INSERT

;----------------------------------------------------------------------------
; string search and replacement
;
(bind-to-key "re-search-forward" "\er")			; INS
(bind-to-key "re-replace-string" "\eq")			; REPL

;----------------------------------------------------------------------------
; window shifts
;
(bind-to-key "next-window" "\e\n")			; ESC-DOWNARROW
(bind-to-key "previous-window" "\e\^K")			; ESC-UPARROW
 
;----------------------------------------------------------------------------
; jumps
;
(bind-to-key "beginning-of-line" "\e\^H")		; ESC-LEFTARROW
(bind-to-key "end-of-line" "\e\^L")			; ESC-RIGHTARROW
 
;----------------------------------------------------------------------------
;Restore stuff trashed by killring load.
;
(defun (killring-hook
	(bind-to-key "kill-char" (onwyse "\eQ" "\eW"))	; see append-next-kill
	(bind-to-key "next-page" (onwyse "\eh" "\eK"))	; see copy-lines
))

;----------------------------------------------------------------------------
; setup soft-key area for half-bright reverse video
;
(send-string-to-terminal "\eA1t\r")

; wyse.ml ends here ----------------------------------------------------------
